/* This is  -*- C++ -*-  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///  \file  abstractCASModule.cpp
///  \brief Implements an abstract class for interfacing with a computer algebra system like GAP or MAGMA
///  \author Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  \date   2010-09-29
///
///  Copyright (C) 2010 Thomas Feulner <thomas.feulner@uni-bayreuth.de>
///  Distributed  under  the  terms  of  the  GNU  General  Public  License (GPL)
///  http://www.gnu.org/licenses/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "abstractCASModule.h"

#include <iostream>
#include <cstdlib>
#include <sys/wait.h>

AbstractCASModule::AbstractCASModule(char* call[])
{
	/* Setup communication pipeline first */
	if(pipe(m_p2c)){
		std::cerr<<"Pipe error!"<<std::endl;
		exit(0);
	}
	if(pipe(m_c2p)){
		std::cerr<<"Pipe error!"<<std::endl;
		exit(0); 
	}

	m_pid=fork();

	if( m_pid == -1)
	{
		std::cerr<<"Fork error. Exiting."<<std::endl;  /* something went wrong */
		exit(0);
	}
	else if(m_pid)
	{
		/* A positive (non-negative) PID indicates the parent process */
		//		dup2(p2c[1],1);		/* Replace stdout with out side of the pipe */
		close(m_p2c[0]);				/* Close unused side of pipe (in side) */
		close(m_c2p[1]);

		m_to.open( boost::iostreams::file_descriptor_sink(m_p2c[1], boost::iostreams::never_close_handle), 1024, 4);
		m_from.open( boost::iostreams::file_descriptor_source(m_c2p[0], boost::iostreams::never_close_handle), 1024, 4);
	}
	else{
		/* A zero PID indicates that this is the child process */
		close(m_p2c[1]);				/* Close unused side of pipe (out side) */
		close(m_c2p[0]);			

		dup2(m_p2c[0],0);			/* Replace stdin with the in side of the pipe */
		dup2(m_c2p[1],1);	
		
		/* Replace the child fork with a new process */
			//			
			if(execvp(call[0], call) == -1)				
			{
				std::cerr<<"execl Error in AbstractCasModule! "<<std::endl;
				exit(1);
			}
	}
}

AbstractCASModule::~AbstractCASModule()
{
    close(m_p2c[1]);                /* Close unused side of pipe (out side) */
    close(m_c2p[0]);
    m_to.close();
    m_from.close();
    waitpid(m_pid, NULL, 0);
    m_pid=-1;
}

